export queue GlueHostOperatingSystemName GlueHostMainMemoryVirtualSize GlueHostMainMemoryRAMSize_Min GlueCEPolicyMaxCPUTime_Max GlueCEPolicyMaxCPUTime_Min

/usr/bin/perl << "__END_OF_PERL_STUFF__"

#
# File            $Id: lsf_local_submit_attributes.pl,v 1.3 2006/08/31 05:50:55 uschwick Exp $
#
# Purpose         parse information from BLAH and interface them to LSF
#
# History
#                 $Log: lsf_local_submit_attributes.pl,v $
#                 Revision 1.3  2006/08/31 05:50:55  uschwick
#                 added yaim plugin for glite logrotation
#
#                 Revision 1.2  2006/08/27 14:45:45  uschwick
#                 improved support for lsf_local_submit_attributes
#
#                 Revision 1.1  2006/08/16 20:53:04  uschwick
#                 package lsf_local_submit_attributes.sh
#
#                 Revision 1.2  2006/08/11 08:12:26  uschwick
#                 remove new features for production use
#
#                 Revision 1.1  2006/07/18 14:15:34  uschwick
#                 glite CE  hacks for CERN first version
#
#
#
use lib '/opt/glite/lib/perl5';
use strict;
use diagnostics;
use Date::Parse;
use File::Basename;
use CommandProxyTools;

# global definitions

# read from STDIN
my $stdin = 0;

# which queue names should be selected. This is used as regexp.
# 
# try to guess the LSF installation path
#
my $binPath;
if ( -e "/usr/bin/lshosts") {
    $binPath = "/usr/bin";
} elsif ( -e "/usr/local/bin/lshosts"){
    $binPath = "/usr/local/bin";
} elsif ( -e "/usr/local/lsf/bin/lshosts") {
    $binPath = "/usr/local/lsf/bin";
} elsif ( -e "/usr/lsf/bin/lshosts") {
    $binPath = "/usr/lsf/bin";
} elsif ( -e "/opt/lsf/bin/lshosts") {
    $binPath = "/opt/lsf/bin";
} else {
    die "cannot find LSF executables. Giving up.";
}

IniCommandProxyTools($binPath);

ReadLCGInfoDynamicLsfConf();
@LsfLocalQueues = split /\s+/, $ENV{"LSF_LOCAL_QUEUES"} if ($ENV{"LSF_LOCAL_QUEUES"});

print @LsfLocalQueues;
  
# verbose output
my $debug = 0;

#------------------- no need to change anything below this line --------------------
# parse STDIN and get options
my %inputs;

# first parse environment variables
foreach my $env (keys %ENV){
    $inputs{$env} = $ENV{$env} if ($env =~ /^Glue/ || $env =~ /VirtualOrganisation/);
    #print "$env $ENV{$env}\n" if ($debug);
}

# original submission queue
my $GotQueueName;
if ($ENV{"queue"}){
    $GotQueueName = $ENV{"queue"};
    chomp $GotQueueName;
}
my $GotQueueResReq;

# read from STDIN overriding the environment settings
if ($stdin){
    my @input = split <>;
    foreach (@input){
          chomp;
          next if /^DEBUG/;
          $inputs{$1}=$2 if /^([\w\d\_]+)\s*=\s*\"?([\w\d\_]+)\"?/;
    }
    
}

my %LocalQueue;

if (@LsfLocalQueues && (defined $GotQueueName)){
# Query the local queues
    my $command = "bqueues -l";
    
    open(BQUEUES,$command . "|");
    my $CurrentQueueName;
    my $cpulimitflag = 0;
    my $runlimitflag = 0;
    while (<BQUEUES>){
	chomp;
	my $line = $_;
	$CurrentQueueName = $1 if ($line =~ /QUEUE:\s+([\w\d\_\-]+)/);
	$GotQueueResReq = $1 if ((defined $CurrentQueueName)&&($CurrentQueueName eq $GotQueueName)&&($line =~ /RES_REQ:\s+(.+)/));
	if ($CurrentQueueName && (0<grep {$CurrentQueueName eq $_} @LsfLocalQueues)){
	    $cpulimitflag = 1 if ($line =~ /CPULIMIT/);
	    $runlimitflag = 1 if ($line =~ /RUNLIMIT/);
	    if ($CurrentQueueName && $cpulimitflag && $line =~ /([\d\.]+)\s+min\s+of/){
		$LocalQueue{$CurrentQueueName}{"CPULimit"} = $1;
		$cpulimitflag = 0;
	    }
	    if ($CurrentQueueName && $runlimitflag && $line =~ /([\d\.]+)\s+min\s+of/){
		$LocalQueue{$CurrentQueueName}{"RUNLimit"} = $1;
		$runlimitflag = 0;
	    }
	}
    }
    close BQUEUES;
    
    
    if ($debug){
	foreach my $key (keys %LocalQueue){
	    next if ($key eq "CPULimit" || $key eq "RUNLimit");
	    print STDERR "$key ".$LocalQueue{$key}{"CPULimit"}." ".$LocalQueue{$key}{"RUNLimit"}."\n";
	}
    }
}

my @select;
my $maxcpu = 0;
my $mincpu = 9999999;
# translate into lsf resource requirements
foreach my $key (keys %inputs){
    print STDERR $key ." => ". $inputs{$key} . "\n" if $debug;
    push @select, "type==$inputs{$key}" if ($key =~ /GlueHostOperatingSystemName/);
    push @select,  "mem>=$inputs{$key}" if ($key =~ /GlueHostMainMemoryRAMSize_Min/);
    push @select,  "swap>=$inputs{$key}" if ($key =~ /GlueHostMainMemoryVirtualSize/);
    if (($key =~ /GlueCEPolicyMaxCPUTime_Max/)&&($inputs{$key} =~ /\d+/)){
	$maxcpu = $inputs{$key};
	print "#BSUB -W ". (1.5*$maxcpu)."\n";
    } 
    $mincpu = $inputs{$key} if (($key =~ /GlueCEPolicyMaxCPUTime_Min/)&&($inputs{$key} =~ /\d+/));
};

if ($GotQueueResReq){
    # submission queue has some resource requirement that needs to be preserved
    push @select, $1 if ($GotQueueResReq =~ /select\[(.+?)\]$/);
}


my $QueueName;

if ((@LsfLocalQueues)  && (defined $GotQueueName) && (($mincpu<9999999)|| ($maxcpu>0))){
# select candidate queue based on the minimal CPU requirement
    my @candidates;
    my $LongestQueue;
    foreach my $key (keys %LocalQueue){
	next if ($key eq "CPULimit" || $key eq "RUNLimit");
	my $QueueCPULimit = $LocalQueue{$key}{"CPULimit"};
	if ($LongestQueue){
	    $LongestQueue = $key if ($QueueCPULimit > $LocalQueue{$LongestQueue}{"CPULimit"});
	} else {
	    $LongestQueue = $key;
	}
	push @candidates, $key if ($mincpu < $QueueCPULimit);
    }
    
# among those we prefer the shortes queue, if the run limit constrain is fulfilled
    if (@candidates){
	$QueueName = shift @candidates;
	foreach my $candidate (@candidates){
	    my $QueueCPULimit = $LocalQueue{$candidate}{"CPULimit"};
	    my $QueueRUNLimit = $LocalQueue{$candidate}{"RUNLimit"};
	    $QueueName = $candidate if (($QueueCPULimit < $LocalQueue{$QueueName}{"CPULimit"})&&($QueueRUNLimit>$LocalQueue{$QueueName}{"CPULimit"}));
	}
    }
# default is that we use the longest queue available
    $QueueName = $LongestQueue if (!$QueueName);
}

# build the resourse string
my $BsubOption = "";
if (@select) {
    print "Select is @select\n";
    $BsubOption = "-R \"select[" . shift @select;
    if (@select) {
        foreach (@select){
            $BsubOption .= "&&".$_;
        }
    }
    $BsubOption .= "]\"";
    print "#BSUB $BsubOption\n";
}
print "#BSUB -q $QueueName\n" if ($QueueName);
#"__END_OF_PERL_STUFF__"


